@@ -66,6 +66,7 @@ def convert(
     not_use_opname_auto_generate: Optional[bool] = False,
     batch_size: Optional[int] = None,
     overwrite_input_shape: Optional[List[str]] = None,
+    output_nms_with_dynamic_tensor: Optional[bool] = False,
     keep_ncw_or_nchw_or_ncdhw_input_names: Optional[List[str]] = None,
     keep_nwc_or_nhwc_or_ndhwc_input_names: Optional[List[str]] = None,
     keep_shape_absolutely_input_names: Optional[List[str]] = None,
@@ -80,16 +81,7 @@ def convert(
     replace_argmax_to_fused_argmax_and_indicies_is_int64: Optional[bool] = False,
     replace_argmax_to_fused_argmax_and_indicies_is_float32: Optional[bool] = False,
     fused_argmax_scale_ratio: Optional[float] = 0.5,
-    replace_asin_to_pseudo_asin: Optional[bool] = False,
-    replace_acos_to_pseudo_acos: Optional[bool] = False,
-    replace_abs_to_pseudo_abs: Optional[bool] = False,
-    replace_prelu_to_pseudo_prelu: Optional[bool] = False,
-    replace_leakyrelu_to_pseudo_leakyrelu: Optional[bool] = False,
-    replace_power_to_pseudo_power: Optional[bool] = False,
-    replace_gathernd_to_pseudo_gathernd: Optional[bool] = False,
-    replace_neg_to_pseudo_neg: Optional[bool] = False,
-    replace_hardswish_to_pseudo_hardswish: Optional[bool] = False,
-    replace_erf_to_pseudo_erf: Optional[bool] = False,
+    replace_to_pseudo_operators: List[str] = None,
     param_replacement_file: Optional[str] = '',
     check_gpu_delegate_compatibility: Optional[bool] = False,
     check_onnx_tf_outputs_elementwise_close: Optional[bool] = False,
@@ -217,6 +209,18 @@ def convert(
         Numerical values other than dynamic dimensions are ignored.\n
         Ignores batch_size if specified at the same time as batch_size.
 
+    output_nms_with_dynamic_tensor: Optional[bool]
+        The number of bounding boxes in the NMS output results is\n
+        not fixed at the maximum number of max_output_boxes_per_class,\n
+        but rather at the smallest possible number of dynamic tensors.\n
+        If this option is disabled, NMS output is padded to the number\n
+        set in the max_output_boxes_per_class attribute.\n
+        e.g.\n
+        disable --output_nms_with_dynamic_tensor:\n
+            output_tensor_shape: [100, 7]\n
+        enable --output_nms_with_dynamic_tensor:\n
+            output_tensor_shape: [N, 7]
+
     keep_ncw_or_nchw_or_ncdhw_input_names: Optional[List[str]]
         Holds the NCW or NCHW or NCDHW of the input shape for the specified INPUT OP names.\n
         If a nonexistent INPUT OP name is specified, it is ignored.\n
@@ -308,35 +312,11 @@ def convert(
         0.0 < fused_argmax_scale_ratio <= 1.0\n
         Default: 0.5
 
-    replace_asin_to_pseudo_asin: Optional[bool]
-        Replace Asin with a pseudo Asin.
-
-    replace_acos_to_pseudo_acos: Optional[bool]
-        Replace Acos with a pseudo Acos.
-
-    replace_abs_to_pseudo_abs: Optional[bool]
-        Replace Abs with a pseudo Abs.
-
-    replace_prelu_to_pseudo_prelu: Optional[bool]
-        Replace PReLU with a pseudo PReLU.
-
-    replace_leakyrelu_to_pseudo_leakyrelu: Optional[bool]
-        Replace LeakyReLU with a pseudo LeakyReLU.
-
-    replace_power_to_pseudo_power: Optional[bool]
-        Replace Power with a pseudo Power.
-
-    replace_gathernd_to_pseudo_gathernd: Optional[bool]
-        Replace GatherND with a pseudo GatherND.
-
-    replace_neg_to_pseudo_neg: Optional[bool]
-        Replace Neg with a pseudo Neg.
-
-    replace_hardswish_to_pseudo_hardswish: Optional[bool]
-        Replace HardSwish with a pseudo HardSwish.
-
-    replace_erf_to_pseudo_erf: Optional[bool]
-        Replace Erf with a pseudo Erf.
+    replace_to_pseudo_operators: List[str]
+        Replace list of operators to pseudo operators. \n
+        Full name of the target operators should be given. \n
+        Currently supported operators :
+        Asin, Acos, Abs, PReLU, LeakyReLU, Power, GatherND, Neg, HardSwish, Erf
 
     mvn_epsilon: Optional[float]
         For MeanVarianceNormalization.\n
@@ -613,29 +593,39 @@ def convert(
     # Generate output for all OPs.
     # Used to verify the output error of each OP in the TensorFlow model.
     full_ops_output_names = []
+    onnx_tensor_infos_for_validation = None
     for graph_node in graph.nodes:
         full_ops_output_names_sub = []
         for graph_node_output in graph_node.outputs:
             full_ops_output_names_sub.append(graph_node_output.name)
         full_ops_output_names.extend(full_ops_output_names_sub)
-    onnx_outputs_for_validation: List[np.ndarray] = dummy_onnx_inference(
-        onnx_graph=onnx_graph,
-        output_names=full_ops_output_names,
-    )
-    """
-    onnx_tensor_infos_for_validation:
-        {
-            onnx_output_name: np.ndarray,
-            onnx_output_name: np.ndarray,
-            onnx_output_name: np.ndarray,
-                        :
+    # Models with errors during inference in onnxruntime skip dummy inference.
+    try:
+        onnx_outputs_for_validation: List[np.ndarray] = dummy_onnx_inference(
+            onnx_graph=onnx_graph,
+            output_names=full_ops_output_names,
+        )
+        """
+        onnx_tensor_infos_for_validation:
+            {
+                onnx_output_name: np.ndarray,
+                onnx_output_name: np.ndarray,
+                onnx_output_name: np.ndarray,
+                            :
+            }
+        """
+        onnx_tensor_infos_for_validation = {
+            ops_output_name: onnx_output_for_validation \
+                for ops_output_name, onnx_output_for_validation \
+                    in zip(full_ops_output_names, onnx_outputs_for_validation)
         }
-    """
-    onnx_tensor_infos_for_validation = {
-        ops_output_name: onnx_output_for_validation \
-            for ops_output_name, onnx_output_for_validation \
-                in zip(full_ops_output_names, onnx_outputs_for_validation)
-    }
+    except Exception as ex:
+        print(
+            f'{Color.YELLOW}WARNING:{Color.RESET} ' +\
+            f'The optimization process for shape estimation is skipped ' +
+            f'because it contains OPs that cannot be inferred by the standard onnxruntime.'
+        )
+        print(f'{Color.YELLOW}WARNING:{Color.RESET} {ex}')
 
     if not non_verbose:
         print('')
@@ -644,6 +634,9 @@ def convert(
     # Create Output folder
     os.makedirs(output_folder_path, exist_ok=True)
 
+    if replace_to_pseudo_operators is None:
+        replace_to_pseudo_operators = []
+
     # Define additional parameters
     additional_parameters = {
         'onnx_graph': onnx_graph,
@@ -659,20 +652,12 @@ def convert(
         'replace_argmax_to_fused_argmax_and_indicies_is_int64': replace_argmax_to_fused_argmax_and_indicies_is_int64,
         'replace_argmax_to_fused_argmax_and_indicies_is_float32': replace_argmax_to_fused_argmax_and_indicies_is_float32,
         'fused_argmax_scale_ratio': fused_argmax_scale_ratio,
-        'replace_asin_to_pseudo_asin': replace_asin_to_pseudo_asin,
-        'replace_acos_to_pseudo_acos': replace_acos_to_pseudo_acos,
-        'replace_abs_to_pseudo_abs': replace_abs_to_pseudo_abs,
-        'replace_prelu_to_pseudo_prelu': replace_prelu_to_pseudo_prelu,
-        'replace_leakyrelu_to_pseudo_leakyrelu': replace_leakyrelu_to_pseudo_leakyrelu,
-        'replace_power_to_pseudo_power': replace_power_to_pseudo_power,
-        'replace_gathernd_to_pseudo_gathernd': replace_gathernd_to_pseudo_gathernd,
-        'replace_neg_to_pseudo_neg': replace_neg_to_pseudo_neg,
-        'replace_hardswish_to_pseudo_hardswish': replace_hardswish_to_pseudo_hardswish,
-        'replace_erf_to_pseudo_erf': replace_erf_to_pseudo_erf,
+        'replace_to_pseudo_operators': replace_to_pseudo_operators,
         'replacement_parameters': replacement_parameters,
         'mvn_epsilon': mvn_epsilon,
         'output_signaturedefs': output_signaturedefs,
         'onnx_tensor_infos_for_validation': onnx_tensor_infos_for_validation,
+        'output_nms_with_dynamic_tensor': output_nms_with_dynamic_tensor,
     }
 
     tf_layers_dict = {}
@@ -1174,78 +1159,88 @@ def convert(
                 elif check_onnx_tf_outputs_sample_data_normalization == "denorm":
                     test_data_nhwc = test_data_nhwc * 255.0
 
-            # ONNX dummy inference
-            dummy_onnx_outputs: List[np.ndarray] = dummy_onnx_inference(
-                onnx_graph=onnx_graph,
-                output_names=ops_output_names,
-                test_data_nhwc=test_data_nhwc,
-            )
-            # TF dummy inference
-            tf_tensor_infos: Dict[Any] = dummy_tf_inference(
-                model=model,
-                inputs=inputs,
-                test_data_nhwc=test_data_nhwc,
-            )
-            # Validation
-            onnx_tensor_infos = {
-                output_name: dummy_onnx_output \
-                    for output_name, dummy_onnx_output in zip(ops_output_names, dummy_onnx_outputs)
-            }
-            """
-            np.allclose(
-                dummy_onnx_outputs,
-                dummy_tf_outputs,
-                rtol=0.0,
-                atol=1e-04,
-                equal_nan=True,
-            )
-
-            check_results: Dict[str, List[np.ndarray, int, float|int]]
-                {
-                    onnx_output_name: [
-                        onnx_tensor,
-                        matched_flg, <--- 0: Unmatched, 1: Matched, 2: Skipped (Deleted or Shape Unmatched)
-                        max_abs_err,
-                    ]
-                }
-            """
-            input_names = [k.name for k in inputs]
-            onnx_tf_output_pairs = {
-                (k, v['tf_node'].name): (onnx_tensor_infos[k], tf_tensor_infos[v['tf_node'].name])
-                    for k, v in tf_layers_dict.items() if k not in input_names and not hasattr(v['tf_node'], 'numpy')
-            }
-
-            check_results = onnx_tf_tensor_validation(
-                output_pairs=onnx_tf_output_pairs,
-                rtol=check_onnx_tf_outputs_elementwise_close_rtol,
-                atol=check_onnx_tf_outputs_elementwise_close_atol,
-            )
-            for (onnx_output_name, tf_output_name), checked_value in check_results.items():
-                validated_onnx_tensor: np.ndarray = checked_value[0]
-                matched_flg: int = checked_value[1]
-                max_abs_err: Any = checked_value[2]
-                message = ''
-                if matched_flg == 0:
-                    message = \
-                        f'{Color.GREEN}validate_result{Color.RESET}: ' +\
-                        f'{Color.REVERCE}{Color.YELLOW} Unmatched {Color.RESET} ' +\
-                        f'{Color.GREEN}max_abs_error{Color.RESET}: {max_abs_err}'
-                elif matched_flg == 1:
-                    message = \
-                        f'{Color.GREEN}validate_result{Color.RESET}: ' +\
-                        f'{Color.REVERCE}{Color.GREEN} Matches {Color.RESET}'
-                elif matched_flg == 2:
-                    message = \
-                        f'{Color.GREEN}validate_result{Color.RESET}: ' +\
-                        f'{Color.REVERCE}{Color.BLUE} Skipped (Deleted or Shape Unmatched) {Color.RESET}'
-                print(
-                    f'{Color.GREEN}INFO:{Color.RESET} '+
-                    f'{Color.GREEN}onnx_output_name{Color.RESET}: {onnx_output_name} '+
-                    f'{Color.GREEN}tf_output_name{Color.RESET}: {tf_output_name} '+
-                    f'{Color.GREEN}shape{Color.RESET}: {validated_onnx_tensor.shape} '+
-                    f'{Color.GREEN}dtype{Color.RESET}: {validated_onnx_tensor.dtype} '+
-                    f'{message}'
+            dummy_onnx_outputs = None
+            try:
+                # ONNX dummy inference
+                dummy_onnx_outputs: List[np.ndarray] = dummy_onnx_inference(
+                    onnx_graph=onnx_graph,
+                    output_names=ops_output_names,
+                    test_data_nhwc=test_data_nhwc,
                 )
+            except Exception as ex:
+                print(
+                    f'{Color.YELLOW}WARNING:{Color.RESET} ' +\
+                    f'The accuracy error measurement process was skipped ' +
+                    f'because the standard onnxruntime contains OPs that cannot be inferred.'
+                )
+                print(f'{Color.YELLOW}WARNING:{Color.RESET} {ex}')
+            else:
+                # TF dummy inference
+                tf_tensor_infos: Dict[Any] = dummy_tf_inference(
+                    model=model,
+                    inputs=inputs,
+                    test_data_nhwc=test_data_nhwc,
+                )
+                # Validation
+                onnx_tensor_infos = {
+                    output_name: dummy_onnx_output \
+                        for output_name, dummy_onnx_output in zip(ops_output_names, dummy_onnx_outputs)
+                }
+                """
+                np.allclose(
+                    dummy_onnx_outputs,
+                    dummy_tf_outputs,
+                    rtol=0.0,
+                    atol=1e-04,
+                    equal_nan=True,
+                )
+
+                check_results: Dict[str, List[np.ndarray, int, float|int]]
+                    {
+                        onnx_output_name: [
+                            onnx_tensor,
+                            matched_flg, <--- 0: Unmatched, 1: Matched, 2: Skipped (Deleted or Shape Unmatched)
+                            max_abs_err,
+                        ]
+                    }
+                """
+                input_names = [k.name for k in inputs]
+                onnx_tf_output_pairs = {
+                    (k, v['tf_node'].name): (onnx_tensor_infos[k], tf_tensor_infos[v['tf_node'].name])
+                        for k, v in tf_layers_dict.items() if k not in input_names and not hasattr(v['tf_node'], 'numpy')
+                }
+
+                check_results = onnx_tf_tensor_validation(
+                    output_pairs=onnx_tf_output_pairs,
+                    rtol=check_onnx_tf_outputs_elementwise_close_rtol,
+                    atol=check_onnx_tf_outputs_elementwise_close_atol,
+                )
+                for (onnx_output_name, tf_output_name), checked_value in check_results.items():
+                    validated_onnx_tensor: np.ndarray = checked_value[0]
+                    matched_flg: int = checked_value[1]
+                    max_abs_err: Any = checked_value[2]
+                    message = ''
+                    if matched_flg == 0:
+                        message = \
+                            f'{Color.GREEN}validate_result{Color.RESET}: ' +\
+                            f'{Color.REVERCE}{Color.YELLOW} Unmatched {Color.RESET} ' +\
+                            f'{Color.GREEN}max_abs_error{Color.RESET}: {max_abs_err}'
+                    elif matched_flg == 1:
+                        message = \
+                            f'{Color.GREEN}validate_result{Color.RESET}: ' +\
+                            f'{Color.REVERCE}{Color.GREEN} Matches {Color.RESET}'
+                    elif matched_flg == 2:
+                        message = \
+                            f'{Color.GREEN}validate_result{Color.RESET}: ' +\
+                            f'{Color.REVERCE}{Color.BLUE} Skipped (Deleted or Shape Unmatched) {Color.RESET}'
+                    print(
+                        f'{Color.GREEN}INFO:{Color.RESET} '+
+                        f'{Color.GREEN}onnx_output_name{Color.RESET}: {onnx_output_name} '+
+                        f'{Color.GREEN}tf_output_name{Color.RESET}: {tf_output_name} '+
+                        f'{Color.GREEN}shape{Color.RESET}: {validated_onnx_tensor.shape} '+
+                        f'{Color.GREEN}dtype{Color.RESET}: {validated_onnx_tensor.dtype} '+
+                        f'{message}'
+                    )
 
         return model
 
@@ -1417,6 +1412,22 @@ def main():
             'Numerical values other than dynamic dimensions are ignored. \n' +
             'Ignores --batch_size if specified at the same time as --batch_size.'
     )
+    parser.add_argument(
+        '-onwdt',
+        '--output_nms_with_dynamic_tensor',
+        action='store_true',
+        help=\
+            'The number of bounding boxes in the NMS output results is \n' +
+            'not fixed at the maximum number of max_output_boxes_per_class, \n' +
+            'but rather at the smallest possible number of dynamic tensors. \n' +
+            'If this option is disabled, NMS output is padded to the number \n' +
+            'set in the max_output_boxes_per_class attribute. \n' +
+            'e.g. \n' +
+            'disable --output_nms_with_dynamic_tensor: \n' +
+            '    output_tensor_shape: [100, 7] \n' +
+            'enable --output_nms_with_dynamic_tensor: \n' +
+            '    output_tensor_shape: [N, 7]'
+    )
     parser.add_argument(
         '-k',
         '--keep_ncw_or_nchw_or_ncdhw_input_names',
@@ -1568,64 +1579,14 @@ def main():
             'Default: 0.5'
     )
     parser.add_argument(
-        '-rasin',
-        '--replace_asin_to_pseudo_asin',
-        action='store_true',
-        help='Replace Asin with a pseudo Asin.'
-    )
-    parser.add_argument(
-        '-racos',
-        '--replace_acos_to_pseudo_acos',
-        action='store_true',
-        help='Replace Acos with a pseudo Acos.'
-    )
-    parser.add_argument(
-        '-rabs',
-        '--replace_abs_to_pseudo_abs',
-        action='store_true',
-        help='Replace Abs with a pseudo Abs.'
-    )
-    parser.add_argument(
-        '-rpr',
-        '--replace_prelu_to_pseudo_prelu',
-        action='store_true',
-        help='Replace PReLU with a pseudo PReLU.'
-    )
-    parser.add_argument(
-        '-rlr',
-        '--replace_leakyrelu_to_pseudo_leakyrelu',
-        action='store_true',
-        help='Replace LeakyReLU with a pseudo LeakyReLU.'
-    )
-    parser.add_argument(
-        '-rpw',
-        '--replace_power_to_pseudo_power',
-        action='store_true',
-        help='Replace Power with a pseudo Power.'
-    )
-    parser.add_argument(
-        '-rgn',
-        '--replace_gathernd_to_pseudo_gathernd',
-        action='store_true',
-        help='Replace GatherND with a pseudo GatherND.'
-    )
-    parser.add_argument(
-        '-rng',
-        '--replace_neg_to_pseudo_neg',
-        action='store_true',
-        help='Replace Neg with a pseudo Neg.'
-    )
-    parser.add_argument(
-        '-rhs',
-        '--replace_hardswish_to_pseudo_hardswish',
-        action='store_true',
-        help='Replace HardSwish with a pseudo HardSwish.'
-    )
-    parser.add_argument(
-        '-rerf',
-        '--replace_erf_to_pseudo_erf',
-        action='store_true',
-        help='Replace Erf with a pseudo Erf.'
+        '-rtpo',
+        '--replace_to_pseudo_operators',
+        nargs="*",
+        default=[],
+        help='Replace list of operators to pseudo operators. \n ' +
+             'Full name of the target operators should be given. \n ' +
+             'Currently supported operators : ' +
+             'Asin, Acos, Abs, PReLU, LeakyReLU, Power, GatherND, Neg, HardSwish, Erf'
     )
     parser.add_argument(
         '-me',
@@ -1750,6 +1711,10 @@ def main():
     if len(calib_params) == 0:
         calib_params = None
 
+    args.replace_to_pseudo_operators = [
+        name.lower() for name in args.replace_to_pseudo_operators
+    ]
+
     # Convert
     model = convert(
         input_onnx_file_path=args.input_onnx_file_path,
@@ -1765,6 +1730,7 @@ def main():
         not_use_opname_auto_generate=args.not_use_opname_auto_generate,
         batch_size=args.batch_size,
         overwrite_input_shape=args.overwrite_input_shape,
+        output_nms_with_dynamic_tensor=args.output_nms_with_dynamic_tensor,
         keep_ncw_or_nchw_or_ncdhw_input_names=args.keep_ncw_or_nchw_or_ncdhw_input_names,
         keep_nwc_or_nhwc_or_ndhwc_input_names=args.keep_nwc_or_nhwc_or_ndhwc_input_names,
         keep_shape_absolutely_input_names=args.keep_shape_absolutely_input_names,
@@ -1779,16 +1745,7 @@ def main():
         replace_argmax_to_fused_argmax_and_indicies_is_int64=args.replace_argmax_to_fused_argmax_and_indicies_is_int64,
         replace_argmax_to_fused_argmax_and_indicies_is_float32=args.replace_argmax_to_fused_argmax_and_indicies_is_float32,
         fused_argmax_scale_ratio=args.fused_argmax_scale_ratio,
-        replace_asin_to_pseudo_asin=args.replace_asin_to_pseudo_asin,
-        replace_acos_to_pseudo_acos=args.replace_acos_to_pseudo_acos,
-        replace_abs_to_pseudo_abs=args.replace_abs_to_pseudo_abs,
-        replace_prelu_to_pseudo_prelu=args.replace_prelu_to_pseudo_prelu,
-        replace_leakyrelu_to_pseudo_leakyrelu=args.replace_leakyrelu_to_pseudo_leakyrelu,
-        replace_power_to_pseudo_power=args.replace_power_to_pseudo_power,
-        replace_gathernd_to_pseudo_gathernd=args.replace_gathernd_to_pseudo_gathernd,
-        replace_neg_to_pseudo_neg=args.replace_neg_to_pseudo_neg,
-        replace_hardswish_to_pseudo_hardswish=args.replace_hardswish_to_pseudo_hardswish,
-        replace_erf_to_pseudo_erf=args.replace_erf_to_pseudo_erf,
+        replace_to_pseudo_operators=args.replace_to_pseudo_operators,
         param_replacement_file=args.param_replacement_file,
         check_gpu_delegate_compatibility=args.check_gpu_delegate_compatibility,
         check_onnx_tf_outputs_elementwise_close=args.check_onnx_tf_outputs_elementwise_close,
