@@ -1,9 +1,11 @@
+import copy
 import random
 random.seed(0)
 import numpy as np
 np.random.seed(0)
 import tensorflow as tf
 import onnx_graphsurgeon as gs
+from typing import List
 from onnx2tf.utils.common_functions import (
     get_replacement_parameter,
     replace_parameter,
@@ -60,6 +62,14 @@ def make_node(
     indices = tf_layers_dict[graph_node_input_2.name]['tf_node'] \
         if isinstance(graph_node_input_2, gs.Variable) else graph_node_input_2
 
+    before_cast_indices = None
+    if isinstance(indices, np.ndarray) and indices.ndim == 1 and indices[0] is not None:
+        if indices[0] >= 0:
+            before_cast_indices = indices[0]
+    elif isinstance(indices, np.ndarray) and indices.ndim == 0 and indices is not None:
+        if indices >= 0:
+            before_cast_indices = int(indices)
+
     shape = graph_node_output.shape
     dtype = graph_node_output.dtype
 
@@ -118,14 +128,65 @@ def make_node(
         **kwargs,
     )
 
+    # Gather + Unsqueeze -> strided_slice
+    # Replace combination of Gather and Unsqueeze
+    # with strided_slice if available
+    consumer_count = 0
+    consumer_nodes: List[gs.Node] = []
+    while True:
+        try:
+            consumer_node =  graph_node.o(consumer_count, 0)
+            consumer_nodes.append(consumer_node)
+            consumer_count += 1
+        except:
+            break
+    unsqueeze_count = 0
+    for consumer_node in consumer_nodes:
+        if consumer_node.op == 'Unsqueeze' \
+            and hasattr(consumer_node, 'attrs') \
+            and 'axes' in consumer_node.attrs \
+            and len(consumer_node.attrs['axes']) == 1 \
+            and consumer_node.attrs['axes'][0] == axis:
+            unsqueeze_count += 1
+
     # Generation of TF OP
-    tf_layers_dict[graph_node_output.name]['tf_node'] = \
-        tf.gather(
-            params=input_tensor,
-            indices=indices,
-            axis=axis,
-            name=graph_node.name,
+    if unsqueeze_count == consumer_count \
+        and before_cast_indices is not None:
+        # Replace
+        ind = before_cast_indices
+        begin_ = [
+            0 if idx != axis else ind \
+                for idx in range(len(input_tensor.shape))
+        ]
+        end_ = [
+            0 if idx != axis else ind + 1 \
+                for idx in range(len(input_tensor.shape))
+        ]
+        begin_mask_ = sum(
+            [
+                2**idx if idx != axis else 0 \
+                    for idx in range(len(input_tensor.shape))
+            ]
         )
+        end_mask_ = begin_mask_
+        tf_layers_dict[graph_node_output.name]['tf_node'] = \
+            tf.strided_slice(
+                input_=input_tensor,
+                begin=begin_,
+                end=end_,
+                begin_mask=begin_mask_,
+                end_mask=end_mask_,
+            )
+        tf_layers_dict[graph_node_output.name]['unnecessary_gather'] = True
+    else:
+        # No-replace
+        tf_layers_dict[graph_node_output.name]['tf_node'] = \
+            tf.gather(
+                params=input_tensor,
+                indices=indices,
+                axis=axis,
+                name=graph_node.name,
+            )
 
     # Post-process transpose
     tf_layers_dict[graph_node_output.name]['tf_node'] = post_process_transpose(
